#include "iodefine.h"

#define	printf	((int (*)(const char *,...))0x00007c7c)
#define	scanf	((int (*)(const char *,...))0x00007cb8)

// ------------------------------------------------------------
#define ACK		1
#define NAK		-1

// ------------------------------------------------------------
// -- TFT t֌W --
//
#define TFTDATA  (*(volatile unsigned short*)0x08000000)
#define TFTCTRL  (*(volatile unsigned short*)0x08000002)
#define _COL_WHITE    (0xFFFF)
#define _COL_RED      (0xF800)
#define _COL_GREEN    (0x07E0)
#define _COL_BLUE     (0x001F)
#define _COL_BLACK    (0x0000)

volatile	unsigned short FrameBuf[320*240];

void TFT_draw_point(unsigned short x_pix, unsigned short y_pix, unsigned short p_color) {
	FrameBuf[y_pix*320+x_pix] = p_color;
}


// FrameBuf[]̃f[^TFT֓]
void TFT_draw_screen(void) {
	int i;

	TFTCTRL = 0x4001;
	for (i = 0; i < (320*240); i++ ) {
		TFTDATA = FrameBuf[i];
  }
}


void TFT_clear(void) {
	int i;

	TFTCTRL = 0x4001;
	for (i = 0; i < (320*240); i++ ) {
		TFTDATA = _COL_WHITE;
		FrameBuf[i] = _COL_WHITE;
	}
}


void TFT_On(void) {
	TFTCTRL = 0x4000;
}


void init_CS2(void) {
	BSC.CS2BCR.LONG = 0x12490400;
	BSC.CS2WCR      = 0x000302C0;
	PFC.PACRL4.BIT.PA15MD = 1;
	PFC.PACRL2.BIT.PA6MD  = 2;
}

// ------------------------------------------------------------
unsigned short font_data[95][32];

// ------------------------------------------------------------
// Jn(start condition)𔭍s
void I2C_start(void)
{
	IIC2.ICCR2.BYTE = (IIC2.ICCR2.BYTE & 0xbf) | 0x80;
	while (!IIC2.ICSR.BIT.TDRE)
		;
}


// ~(stop condition)𔭍s
void I2C_stop(void)
{
	IIC2.ICSR.BIT.STOP = 0;
	IIC2.ICCR2.BYTE &= 0x3f;
	while (!IIC2.ICSR.BIT.STOP)
		;
}


char I2C_tx_byte(char data)
{
	IIC2.ICDRT = data;
	while (!IIC2.ICSR.BIT.TDRE)
		;
	if (IIC2.ICIER.BIT.ACKBR)
		return (NAK);
	else
		return (ACK);
}


// ------------------------------------------------------------
// EEPROMւ̃oCg
void I2C_EEwrite_byte(unsigned short adr, unsigned char data)
{
	char ack, ret, tmp;

	while (IIC2.ICCR2.BIT.BBSY)
		// BBSY == 1̊
		;

	// }X^M[hɐݒ
	IIC2.ICCR1.BIT.MST = 1;			// }X^[h
	IIC2.ICCR1.BIT.TRS = 1;			// M[h

	do {
		I2C_start();				// Jn𔭍s
		I2C_tx_byte(0xa0);			// Control byte𑗐M
		while (!IIC2.ICSR.BIT.TEND)
			;
		ack = IIC2.ICIER.BIT.ACKBR;
	} while (ack == 1);
	I2C_tx_byte((adr >> 8) & 0xff);	// AhX
	I2C_tx_byte(adr & 0xff);		// AhX
	I2C_tx_byte(data);				// ݃f[^
	while (!IIC2.ICSR.BIT.TEND)
		;

	I2C_stop();						// ~s

	// X[uM[hɖ߂Ă
	IIC2.ICCR1.BIT.MST = 0;			// X[u[h
	IIC2.ICCR1.BIT.TRS = 0;			// M[h

//	IIC2.ICSR.BIT.TEND = 0;		////////////
	IIC2.ICSR.BIT.TDRE = 0;		////////////
}


// EEPROMPoCgǂݏo
unsigned char I2C_EEread_byte(unsigned short adr)
{
	char ack, ret, tmp;

	while (IIC2.ICCR2.BIT.BBSY)
		// BBSY == 1̊
		;

	// }X^M[hɐݒ
	IIC2.ICCR1.BIT.MST = 1;			// }X^[h
	IIC2.ICCR1.BIT.TRS = 1;			// M[h

	do {
		I2C_start();				// Jn𔭍s
		I2C_tx_byte(0xa0);			// Control byte𑗐M
		while (!IIC2.ICSR.BIT.TEND)
			;
		ack = IIC2.ICIER.BIT.ACKBR;
	} while (ack == 1);
	I2C_tx_byte((adr >> 8) & 0xff);	// AhX
	I2C_tx_byte(adr & 0xff);		// AhX
	while (!IIC2.ICSR.BIT.TEND)
		;

	I2C_start();
	I2C_tx_byte(0xa1);				// Control byte𑗐M
	while (!IIC2.ICSR.BIT.TEND)
		;
	IIC2.ICSR.BIT.TEND = 0;
	IIC2.ICCR1.BIT.TRS = 0;			// M[h
	IIC2.ICSR.BIT.TDRE = 0;

	IIC2.ICIER.BIT.ACKBT = 1;		// ԓACK = 1 .. I
	IIC2.ICCR1.BIT.RCVD = 1;		// ŏIf[^
	tmp = IIC2.ICDRR;				// _~[[h

	while (!IIC2.ICSR.BIT.RDRF)
		// M҂
		;

	I2C_stop();						// ~𔭍s
	ret = IIC2.ICDRR;				// Mf[^
	IIC2.ICCR1.BIT.RCVD = 0;

	// X[uM[hɖ߂Ă
	IIC2.ICCR1.BIT.MST = 0;			// X[u[h
	IIC2.ICCR1.BIT.TRS = 0;			// M[h

	return (ret);
}


// ------------------------------------------------------------
void TFT_putch(unsigned short x, unsigned short y, char ch, unsigned short fg_color, unsigned short bg_color)
{
	int i, j;

	for (i = 0; i < 32; i++) {
		for (j = 0; j < 16; j++) {
		}
	}
}


TFT_putstr(unsigned short x, unsigned short y, char *str, unsigned short fg_color, unsigned short bg_color)
{
	char ch;

	while (ch = *str++) {
	}
}


// ------------------------------------------------------------
// ------------------------------------------------------------
void main()
{
	int i, j, code;
	unsigned int adr;
	unsigned short tmp;

	PFC.PBCRL1.BIT.PB3MD = 4;		// PB3SDA[qɐݒ
	PFC.PBCRL1.BIT.PB2MD = 4;		// PB2SCL[qɐݒ

	STB.CR3.BIT._IIC2 = 0;			// X^oC
	IIC2.ICCR1.BIT.ICE = 1;			// I2CoXL
	IIC2.ICCR1.BIT.CKS = 3;			// 313kHz

	// EEPROMfont_dataǂݏo
	adr = 0;
	for (i = 0; i < 95; i++) {
		for (j = 0; j < 32; j++) {
			tmp = I2C_EEread_byte(adr++);
			tmp <<= 8;
			tmp |= I2C_EEread_byte(adr++);
			font_data[i][j] = tmp;
		}
	}

	init_CS2();
	TFT_On();
	TFT_clear();

	TFT_putch(0, 0, 'A', _COL_WHITE, _COL_BLACK);
	TFT_putstr(0, 32, "abcdefg", _COL_BLACK, _COL_WHITE);
	TFT_draw_screen();

	while (1)
		;
}